@@ -32,7 +32,7 @@ def _convert_brreg_date(date: str | None):
     return converted_date
 
 
-def _build_update_struct(data: dict):
+def _build_update_struct_from_brreg_json(data: dict):
     """returns Update struct"""
     update = Update(
         data["organisasjonsnummer"],
@@ -47,6 +47,7 @@ def _build_unit_record(update: Update, fact: dict, download_date: datetime):
     NB: may delete field in fact"""
     fact.pop("_links", None)
     fact.get("organisasjonsform", {}).pop("_links", None)
+    fact.pop("links", None)
     record = {}
     record["organisasjonsnummer"] = update.orgnr
     record["endringstype"] = update.change
@@ -71,10 +72,11 @@ def _get_unit_record_from_update_single(update: Update, unit_name: str, download
 
 def _get_unit_record_from_update_batch(orgnr_update_lkp: dict[str, Update], unit_name: str, download_date: datetime, page_size: int = 500):
     """Returns {orgnr: record} constructed by searching for details for orgs in given {orgnr: update}
-    NB: the dict of updates may have elements removed.
+    NB: if an orgnr is found, it is popped from the orgnr_update_lkp input dictionary
     Org details will not be found if the org has been deleted at any point in the past,
     as this seems to remove them from the search endpoint.
     """
+    logging.info(f"Making search request for detail/fact with orgnr param size: {len(orgnr_update_lkp)}")
     url = _BASE_URL + "/" + unit_name
     params = {
         "organisasjonsnummer": ",".join(orgnr for orgnr in orgnr_update_lkp),
@@ -93,15 +95,31 @@ def _get_unit_record_from_update_batch(orgnr_update_lkp: dict[str, Update], unit
     return orgnr_record_lkp
 
 
-def get_latest_records_for_unit(download_date: datetime, last_modified_date: datetime, unit_name: str, page_size=500) -> Generator[list, None, None]:
+def get_latest_records_for_unit(
+        download_date: datetime,
+        last_modified_date: datetime,
+        unit_name: str, page_size=500,
+        orgnr_search: list[str] | None = None,
+) -> Generator[list, None, None]:
     """params:
         last_modified_date: smallest update date
         unit_name: enheter | undereneheter
-    returns list of records"""
+        orgnr: list[str] | None
+    returns list of records
+    
+    If specified, orgnr is a list of orgnrs,
+    and only these will be searced for. Every orgnr given this way
+    must be associated with the given unit_name.
+    NOTE: when searching for missing orgnr this way, it is recommended
+    to set last_modified_date far back into the past so that you are
+    guaranteed to find at least one update newer than it.
+    """
     url = _BASE_URL + "/oppdateringer/" + unit_name
     headers = _build_headers("oppdatering." + unit_name)
     list_key = "oppdaterte" + unit_name.capitalize()
     params = {"dato": last_modified_date.isoformat(timespec="milliseconds") + "Z"}
+    if orgnr_search is not None:
+        params |= {"organisasjonsnummer": ",".join(orgnr_search)}
     orgnr_record_lkp: dict = {} # {orgnr from latest update for that orgnr: latest fact for that orgnr}
 
     try:
@@ -109,17 +127,16 @@ def get_latest_records_for_unit(download_date: datetime, last_modified_date: dat
     except PagedRequestError:
         logging.warning("Using unpaged request, as paged request will fail due to BRREG API limitations")
         iter_obj = UnpagedRequestForEmbeddedList(url, params, headers, {}, size=page_size, list_key=list_key, timeout=100) # type: ignore
-    
+
     for updates in iter_obj:
         # only keep the latest updates for each orgnr
         # since updates is already sorted with ascending update id, we can just overwrite
-        orgnr_update_lkp = {update.orgnr: update for update in map(_build_update_struct, updates)}
+        orgnr_update_lkp = {update.orgnr: update for update in map(_build_update_struct_from_brreg_json, updates)}
 
         old_length = len(orgnr_update_lkp)
         orgnr_record_lkp_batch = _get_unit_record_from_update_batch(orgnr_update_lkp, unit_name, download_date)
         new_length = len(orgnr_update_lkp)
-        logging.info(f"found {old_length - new_length} of {old_length} orgs with batched search")
-        logging.info("appending remaining using single requests")
+        logging.info(f"found {new_length} / {old_length} orgs with batched search, remaining will be appened using single requests")
         orgnr_record_lkp_batch |= {
             orgnr: _get_unit_record_from_update_single(update, unit_name, download_date)
             for orgnr, update in orgnr_update_lkp.items()
@@ -127,16 +144,30 @@ def get_latest_records_for_unit(download_date: datetime, last_modified_date: dat
 
         orgnr_record_lkp |= orgnr_record_lkp_batch
 
+    # handle orgnr we didn't find any updates for
+    # possibly because it was before update types were introduced, or it has been deleted..
+    if orgnr_search is not None:
+        missing_orgnrs = set(orgnr_search) - orgnr_record_lkp.keys()
+        if len(missing_orgnrs) > 0:
+            logging.warning(f"Found no updates for {len(missing_orgnrs)} requested orgnrs. Facts will be downloaded directly, updates set to unknown")
+            orgnr_record_lkp |= {
+                orgnr: _get_unit_record_from_update_single(
+                            Update(orgnr, "UKJENT", last_modified_date),
+                            unit_name, download_date,
+                        )
+                for orgnr in missing_orgnrs
+            }
+        else:
+            logging.info("All requested orgnrs found.")
     yield list(orgnr_record_lkp.values())
 
 
-def stream_latest_records_for_unit_from_file(download_date: datetime, unit_name: str, batch_size: int = 1000) -> Generator[list, None, None]:
+def stream_latest_records_for_unit_from_file(download_date: datetime, last_modified_date: datetime, unit_name: str, batch_size: int = 1000) -> Generator[list, None, None]:
     """
     yields [records] from large file, must be run some time after 5 brreg local time
     useful for init loading table
     """
-    assert datetime.today().hour > 5, "too early"
-    updated_date_str = datetime.today().replace(hour=5, minute=0, second=0).isoformat(timespec="milliseconds")+"Z"
+    #updated_date_str = last_modified_date.isoformat(timespec="milliseconds")+"Z"
     url = _BASE_URL + "/" + unit_name + "/" + "lastned"
     headers = _build_headers(unit_name, "gzip")
     logging.info("requesting filestream from api")
@@ -149,12 +180,10 @@ def stream_latest_records_for_unit_from_file(download_date: datetime, unit_name:
             for record in ijson.items(file, "item"):  # type: ignore
                 records.append(
                     _build_unit_record(
-                        _build_update_struct(
-                            {
-                                "organisasjonsnummer": record["organisasjonsnummer"],
-                                "endringstype": "INIT",
-                                "dato": updated_date_str
-                            }
+                        Update(
+                            record["organisasjonsnummer"],
+                            "FLATFIL",
+                            last_modified_date,
                         ),
                         record,
                         download_date,
